window.Gleam.initSearch([{"doc":"eventsourcing","title":"eventsourcing","content":"&lt;h1 align=&quot;center&quot;&gt;Eventsourcing&lt;/h1&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  ✨ &lt;strong&gt;Event Sourcing Library for Gleam&lt;/strong&gt; ✨\n&lt;/div&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  A Gleam library for building event-sourced systems.\n&lt;/div&gt;\n\n&lt;br /&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;a href=&quot;https://hex.pm/packages/eventsourcing&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/hexpm/v/eventsourcing&quot;\n      alt=&quot;Available on Hex&quot; /&gt;\n  &lt;/a&gt;\n  &lt;a href=&quot;https://hexdocs.pm/eventsourcing&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/badge/hex-docs-ffaff3&quot;\n      alt=&quot;Documentation&quot; /&gt;\n  &lt;/a&gt;\n&lt;/div&gt;\n\n---\n\n## Table of contents\n\n- [Introduction](#introduction)\n- [Features](#features)\n- [Example](#example)\n  - [Command Handling](#command-handling)\n  - [Event Application](#event-application)\n  - [Running the Example](#running-the-example)\n- [Philosophy](#philosophy)\n- [Installation](#installation)\n- [Support](#support)\n- [Contributing](#contributing)\n- [License](#license)\n\n## Introduction\n\nEventsourcing is a Gleam library designed to help developers build event-sourced systems. Event sourcing is a pattern where changes to the application&#39;s state are stored as a sequence of events. This allows for better traceability, easier debugging, and the ability to recreate the application&#39;s state at any point in time.\n\n## Features\n\n- **Event Sourcing**: Build systems based on the event sourcing pattern.\n- **In-memory Store**: Simple in-memory event store for development and testing.\n- **Command Handling**: Handle commands and produce events.\n- **Event Application**: Apply events to update aggregates.\n\n## Example\n\n### Command Handling\n\n```gleam\nimport eventsourcing\nimport eventsourcing/memory_store\n\nimport gleam/result\n\npub const bank_account_event_type = &quot;BankAccountEvent&quot;\npub const bank_account_type = &quot;BankAccount&quot;\n\npub type BankAccount {\n  BankAccount(balance: Float)\n  UnopenedBankAccount\n}\n\npub type BankAccountCommand {\n  OpenAccount(account_id: String)\n  DepositMoney(amount: Float)\n  WithDrawMoney(amount: Float)\n}\n\npub type BankAccountEvent {\n  AccountOpened(account_id: String)\n  CustomerDepositedCash(amount: Float, balance: Float)\n  CustomerWithdrewCash(amount: Float, balance: Float)\n}\n\npub type BankAccountError {\n  CantDepositNegativeAmount\n  CantOperateOnUnopenedAccount\n  CantWithdrawMoreThanCurrentBalance\n}\n\npub fn handle(\n  bank_account: BankAccount,\n  command: BankAccountCommand,\n) -&gt; Result(List(BankAccountEvent), BankAccountError) {\n  case bank_account, command {\n    UnopenedBankAccount, OpenAccount(account_id) -&gt;\n      Ok([AccountOpened(account_id)])\n    BankAccount(balance), DepositMoney(amount) -&gt; {\n      let balance = balance +. amount\n      case amount &gt;. 0.0 {\n        True -&gt; Ok([CustomerDepositedCash(amount:, balance:)])\n        False -&gt; Error(CantDepositNegativeAmount)\n      }\n    }\n    BankAccount(balance), WithDrawMoney(amount) -&gt; {\n      let balance = balance -. amount\n      case amount &gt;. 0.0 &amp;&amp; balance &gt;. 0.0 {\n        True -&gt; Ok([CustomerWithdrewCash(amount:, balance:)])\n        False -&gt; Error(CantWithdrawMoreThanCurrentBalance)\n      }\n    }\n    _, _ -&gt; Error(CantOperateOnUnopenedAccount)\n  }\n}\n```\n\n### Event Application\n\n```gleam\npub fn apply(bank_account: BankAccount, event: BankAccountEvent) {\n  case bank_account, event {\n    UnopenedBankAccount, AccountOpened(_) -&gt; BankAccount(0.0)\n    BankAccount(_), CustomerDepositedCash(_, balance) -&gt; BankAccount(balance:)\n    BankAccount(_), CustomerWithdrewCash(_, balance) -&gt; BankAccount(balance:)\n    _, _ -&gt; panic\n  }\n}\n```\n\n### Running the Example\n\n```gleam\npub fn main() {\n  let mem_store =\n    memory_store.new(UnopenedBankAccount, handle, apply)\n  let query = fn(\n    aggregate_id: String,\n    events: List(eventsourcing.EventEnvelop(BankAccountEvent)),\n  ) {\n    io.println_error(\n      &quot;Aggregate Bank Account with ID: &quot;\n      &lt;&gt; aggregate_id\n      &lt;&gt; &quot; commited &quot;\n      &lt;&gt; events |&gt; list.length |&gt; int.to_string\n      &lt;&gt; &quot; events.&quot;,\n    )\n  }\n  let event_sourcing = eventsourcing.new(mem_store, [query])\n  eventsourcing.execute(\n    event_sourcing,\n    &quot;92085b42-032c-4d7a-84de-a86d67123858&quot;,\n    OpenAccount(&quot;92085b42-032c-4d7a-84de-a86d67123858&quot;),\n  )\n}\n```\n\n## Philosophy\n\nEventsourcing is designed to make building event-sourced systems easy and intuitive. It encourages a clear separation between command handling and event application, making your code more maintainable and testable.\n\n## Installation\n\nEventsourcing is published on [Hex](https://hex.pm/packages/eventsourcing)! You can add it to your Gleam projects from the command line:\n\n```sh\ngleam add eventsourcing\n```\n\n## Support\n\nEventsourcing is built by [Renatillas](https://github.com/renatillas). Contributions are very welcome! If you&#39;ve spotted a bug, or would like to suggest a feature, please open an issue or a pull request.\n\n## Contributing\n\nContributions are welcome! Please follow these steps:\n\n1. Fork the repository.\n2. Create a new branch (`git checkout -b my-feature-branch`).\n3. Make your changes and commit them (`git commit -m &#39;Add new feature&#39;`).\n4. Push to the branch (`git push origin my-feature-branch`).\n5. Open a pull request.\n\nPlease ensure your code adheres to the project&#39;s coding standards and includes appropriate tests.\n\n## License\n\nThis project is licensed under the MIT License. See the [LICENSE](LICENSE) file for details.\n","url":"index.html"},{"doc":"eventsourcing","title":"AggregateId","content":"pub type AggregateId =\n  String\n\n\nSynonyms:\neventsourcing.AggregateId\neventsourcing AggregateId","url":"eventsourcing.html#AggregateId"},{"doc":"eventsourcing","title":"EventEnvelop","content":"pub type EventEnvelop(event) {\n  MemoryStoreEventEnvelop(\n    aggregate_id: AggregateId,\n    sequence: Int,\n    payload: event,\n    metadata: List(#(String, String)),\n  )\n  SerializedEventEnvelop(\n    aggregate_id: AggregateId,\n    sequence: Int,\n    payload: event,\n    metadata: List(#(String, String)),\n    event_type: String,\n    event_version: String,\n    aggregate_type: String,\n  )\n}\n An EventEnvelop is a wrapper around your domain events\n used by the Event Stores. You can use this type constructor\n if the event store provides a `load_events` function.\nMemoryStoreEventEnvelop(\n  aggregate_id: AggregateId,\n  sequence: Int,\n  payload: event,\n  metadata: List(#(String, String)),\n)\n\n\nSerializedEventEnvelop(\n  aggregate_id: AggregateId,\n  sequence: Int,\n  payload: event,\n  metadata: List(#(String, String)),\n  event_type: String,\n  event_version: String,\n  aggregate_type: String,\n)\n\n\nSynonyms:\neventsourcing.EventEnvelop\neventsourcing EventEnvelop","url":"eventsourcing.html#EventEnvelop"},{"doc":"eventsourcing","title":"EventSourcing","content":"pub opaque type EventSourcing(\n  eventstore,\n  entity,\n  command,\n  event,\n  error,\n  aggregatecontext,\n)\n The main record of the library. \n It holds everything together and serves as a reference point \n for other functions such as execute, load_aggregate_entity, and load_events.\n\nSynonyms:\neventsourcing.EventSourcing\neventsourcing EventSourcing","url":"eventsourcing.html#EventSourcing"},{"doc":"eventsourcing","title":"EventSourcingError","content":"pub type EventSourcingError(domainerror) {\n  DomainError(domainerror)\n  ImplementationError\n  EntityNotFound\n}\n\nDomainError(domainerror)\n\n\nImplementationError\n\n\nEntityNotFound\n\n\nSynonyms:\neventsourcing.EventSourcingError\neventsourcing EventSourcingError","url":"eventsourcing.html#EventSourcingError"},{"doc":"eventsourcing","title":"EventStore","content":"pub type EventStore(eventstore, entity, command, event, error) {\n  EventStore(\n    eventstore: eventstore,\n    load_aggregate: fn(eventstore, AggregateId) -&gt;\n      AggregateContext(entity, command, event, error),\n    load_events: fn(eventstore, AggregateId) -&gt;\n      List(EventEnvelop(event)),\n    commit: fn(\n      eventstore,\n      AggregateContext(entity, command, event, error),\n      List(event),\n      List(#(String, String)),\n    ) -&gt;\n      List(EventEnvelop(event)),\n  )\n}\n Wrapper around the event store implementations\nEventStore(\n  eventstore: eventstore,\n  load_aggregate: fn(eventstore, AggregateId) -&gt;\n    AggregateContext(entity, command, event, error),\n  load_events: fn(eventstore, AggregateId) -&gt;\n    List(EventEnvelop(event)),\n  commit: fn(\n    eventstore,\n    AggregateContext(entity, command, event, error),\n    List(event),\n    List(#(String, String)),\n  ) -&gt;\n    List(EventEnvelop(event)),\n)\n\n\nSynonyms:\neventsourcing.EventStore\neventsourcing EventStore","url":"eventsourcing.html#EventStore"},{"doc":"eventsourcing","title":"add_query","content":"pub fn add_query(\n  eventsouring eventsourcing: EventSourcing(a, b, c, d, e, f),\n  query query: fn(String, List(EventEnvelop(d))) -&gt; Nil,\n) -&gt; EventSourcing(a, b, c, d, e, g)\n\nSynonyms:\neventsourcing.add_query\neventsourcing add_query","url":"eventsourcing.html#add_query"},{"doc":"eventsourcing","title":"execute","content":"pub fn execute(\n  event_sourcing event_sourcing: EventSourcing(a, b, c, d, e, f),\n  aggregate_id aggregate_id: String,\n  command command: c,\n) -&gt; Result(Nil, EventSourcingError(e))\n **Execute**\n The main function of the package. \n Run execute with your event_sourcing instance and the command you want to apply.\n It will return a Result with Ok(Nil) or Error(your domain error) if the command failed.\nSynonyms:\neventsourcing.execute\neventsourcing execute","url":"eventsourcing.html#execute"},{"doc":"eventsourcing","title":"execute_with_metadata","content":"pub fn execute_with_metadata(\n  event_sourcing event_sourcing: EventSourcing(a, b, c, d, e, f),\n  aggregate_id aggregate_id: String,\n  command command: c,\n  metadata metadata: List(#(String, String)),\n) -&gt; Result(Nil, EventSourcingError(e))\n\nSynonyms:\neventsourcing.execute_with_metadata\neventsourcing execute_with_metadata","url":"eventsourcing.html#execute_with_metadata"},{"doc":"eventsourcing","title":"load_aggregate","content":"pub fn load_aggregate(\n  eventsourcing eventsourcing: EventSourcing(a, b, c, d, e, f),\n  aggregate_id aggregate_id: String,\n) -&gt; b\n\nSynonyms:\neventsourcing.load_aggregate\neventsourcing load_aggregate","url":"eventsourcing.html#load_aggregate"},{"doc":"eventsourcing","title":"load_events","content":"pub fn load_events(\n  eventsourcing eventsourcing: EventSourcing(a, b, c, d, e, f),\n  aggregate_id aggregate_id: String,\n) -&gt; List(EventEnvelop(d))\n\nSynonyms:\neventsourcing.load_events\neventsourcing load_events","url":"eventsourcing.html#load_events"},{"doc":"eventsourcing","title":"new","content":"pub fn new(\n  event_store: EventStore(a, b, c, d, e),\n  queries: List(fn(String, List(EventEnvelop(d))) -&gt; Nil),\n) -&gt; EventSourcing(a, b, c, d, e, f)\n Create a new EventSourcing instance providing \n an Event Store and a list of queries you want\n run whenever events are commited.\n\nSynonyms:\neventsourcing.new\neventsourcing new","url":"eventsourcing.html#new"},{"doc":"eventsourcing","title":"eventsourcing","content":"","url":"eventsourcing.html"}]);